#include "1euroFilter.h"

static OneEuroFilter f1;
static OneEuroFilter f2;
static OneEuroFilter f3;

//Filter constants
#define FREQUENCY   60   // [Hz] set by timestamps passed to filter
#define MINCUTOFF   1.0   // [Hz] needs to be tuned according to your application
#define BETA        0.1   // needs to be tuned according to your application
#define ALPHA       0.01

#define OUTPUT_PIN  1   //output to printer, low is triggered state.
#define TOUCH_PIN   7 


#define RGB_BRIGHTNESS 10 // Change white brightness (max 255)
#define RGB_BUILTIN 21

int touchValue;
int lastTouch=0;

uint64_t lastWrite=0;
uint64_t lastLatch=0;

float threshold = 10000.0;

int triggered=0;

float touchFiltered=0.0;

double sloppyLPF=8.0;



void setup(){
  Serial.begin(115200);

  f1.begin(FREQUENCY, MINCUTOFF, BETA);

  pinMode(OUTPUT_PIN, OUTPUT);
  digitalWrite(OUTPUT_PIN, HIGH);


  neopixelWrite(RGB_BUILTIN,RGB_BRIGHTNESS,RGB_BRIGHTNESS,RGB_BRIGHTNESS); // White
  delay(200);
  neopixelWrite(RGB_BUILTIN,RGB_BRIGHTNESS,0,0); // Red
  delay(200);
  neopixelWrite(RGB_BUILTIN,0,RGB_BRIGHTNESS,0); // Green
  delay(200);
  neopixelWrite(RGB_BUILTIN,0,0,RGB_BRIGHTNESS); // Blue
  delay(200);

  //Defaults seem to be 0x1000 and 0x1000 for ESP32
  //ESP32S3 TOUCH_PAD_SLEEP_CYCLE_DEFAULT & TOUCH_PAD_MEASURE_CYCLE_DEFAULT
  //https://github.com/espressif/arduino-esp32/blob/master/cores/esp32/esp32-hal-touch.h
  //#define TOUCH_PAD_SLEEP_CYCLE_DEFAULT   (0xf)   /*!<The number of sleep cycle in each measure process of touch channels.
  //#define TOUCH_PAD_MEASURE_CYCLE_DEFAULT (500)   /*!<The times of charge and discharge in each measure process of touch channels.
   
  //0x200 should be 8 times quicker response
  touchSetCycles(0x40,0x0); //try brief dead period after measure cycle to test drift  

  //touchAttachInterrupt(TOUCH_PIN, touchTriggered, threshold);  

}

void loop(){
  touchValue = touchRead(TOUCH_PIN);
  // touchFiltered = f1.filter(touchValue);
  //ctr++;
  //write value every second or when it changes
  
  sloppyLPF = (0.1 * touchValue) + (0.9 * sloppyLPF);

  // if(touchValue != lastTouch || millis() - lastWrite > 1000) {
  //   Serial.print(touchValue, DEC);
  //   Serial.printf(",%d",ctr);
  //   Serial.print('\n');
  //   lastTouch=touchValue;
  //   lastWrite=millis();
  //   ctr=0;  //track how many samples per second
  // }

  if( millis() - lastWrite > 100  || touchValue != lastTouch) {
    Serial.printf("%d",touchValue);
    // Serial.printf(",%f",sloppyLPF);
    // Serial.printf(",%d",lastLatch);    
    Serial.print('\n');
    lastWrite=millis();
    //ctr=0;  //track how many samples per second
  }

    lastTouch=touchValue;

  //Binary output to the printer & led feedback
  if(touchValue > threshold && !triggered)  {
      neopixelWrite(RGB_BUILTIN,RGB_BRIGHTNESS,0,0); // Red
      digitalWrite(OUTPUT_PIN, LOW);
      lastLatch = millis();
      // Serial.print("triggered\n");
      triggered=1;
  } else if( sloppyLPF <= threshold && (millis() - lastLatch) > 50 && triggered) {
    neopixelWrite(RGB_BUILTIN,0,RGB_BRIGHTNESS,0); // Green
    digitalWrite(OUTPUT_PIN, HIGH);
    // Serial.print("unlatched\n");
    triggered=0;
  }


}








